<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
                xmlns:html="http://www.w3.org/1999/xhtml"
                xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
                xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources"
                xmlns:xslLib="http://www.lefebvre-sarrut.eu/ns/els/xslLib"
                xmlns:functx="http://www.functx.com"
                xmlns:fn="http://www.w3.org/2005/xpath-functions"
                exclude-result-prefixes="#all"
                version="3.0">
  
  <!--+==================================================+
      | INIT.                                            |
      +==================================================+-->
  
  <!-- IMPORTS -->
  <xsl:import href="../../xslLib/xslLib/els-common.xsl"/>
  <xsl:import href="../../xslLib/xslLib/els-log.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Stylesheet that can be used to apply a TR pattern file to a sample document containing strings that can be matched by the pattern file regular expressions.</xd:p>
      <xd:p>These TR pattern files are used by the editor to automatically detect and qualify links (belonging to ECM TRs).</xd:p>
      <xd:p>The TR patterns follow this format:</xd:p>
      <xd:pre>/((Arrêt)\s(?:du)\s?(?:[0-9]+[-/][0-9]+[-/][0-9]+)\s(?:n°|no)\s?([0-9]+-[0-9]+))/gi|||['data-xml-nature','data-xml-date','data-xml-ref']</xd:pre>
      <xd:p>where:</xd:p>
      <xd:ul>
        <xd:li>the regular expression is enclosed between two / / characters;</xd:li>
        <xd:li>the whole link to match is inside parenthesis ( ) characters;</xd:li>
        <xd:li>there can be contextual bits before and after the link;</xd:li>
        <xd:li>flags can be specified after the closing / character;</xd:li>
        <xd:li>the link attribute names are specified inside brackets [ ] after the separator |||;</xd:li>
        <xd:li>there must be a regex capturing group for each attribut name.</xd:li>
      </xd:ul>
      <xd:p>The pattern file to apply is specified by the <xd:ref name="xfRes:TR_patternFileUri" type="parameter">$xfRes:TR_patternFileUri</xd:ref> parameter.</xd:p>
      <xd:p>The pattern file is first parsed and compiled to an XML format; each regular expression it holds is then applied to the input document (in text mode).</xd:p>
      <xd:p>The regular expressions are applied in the pattern file order. Once a substring matches a regular expression, it is no longer parsed.</xd:p>
      <xd:p>This stylesheet can be used to build unit tests for the TR pattern files.</xd:p>
    </xd:desc>
  </xd:doc>
  
  <!-- PARAMS / VARIABLES -->
  <!-- Mandatory / URI of the TR pattern file containing the editor regular expressions to use -->
  <xsl:param name="xfRes:TR_patternFileUri" as="xs:anyURI"/>
  <!-- Write intermediary debug files -->
  <xsl:param name="xfRes:debug" select="false()" as="xs:boolean"/>
  <!-- For log / debug purposes -->
  <xsl:variable name="resName" select="static-base-uri()" as="xs:anyURI"/>
  
  <!-- OUTPUT -->
  <xsl:output encoding="UTF-8" indent="no" method="xml" omit-xml-declaration="no"/>
  
  <xd:doc>
    <xd:desc>A way to launch this XSL without mode.</xd:desc>
  </xd:doc>
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:compilePatternsAndApplyRegex"/>
  </xsl:template>
  
  
  <!--+==================================================+
      | MODE xfRes:compilePatternsAndApplyRegex          |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Launches the transformation.</xd:p>
      <xd:p>INPUT: HTML file as generated by FLASH editor, containing instances matched by the patterns.</xd:p>
      <xd:ul>
        <xd:li>The input file is re-loaded with fn:unparsed-text() (which is not really optimized but necessary to get its content as text).</xd:li>
        <xd:li>The pattern file is loaded (from <xd:ref name="xfRes:TR_patternFileUri" type="parameter">$xfRes:TR_patternFileUri</xd:ref>) with fn:unparsed-text-lines().</xd:li>
      </xd:ul>
      <xd:p>The transformation operates in 2 steps:</xd:p>
      <xd:ul>
        <xd:li>STEP1: Compile the patterns from the file ($xfRes:TR_patternFileUri)</xd:li>
        <xd:li>STEP2: Apply the regular expressions on the input file</xd:li>
      </xd:ul>
    </xd:desc>
  </xd:doc>
  <xsl:template match="/" mode="xfRes:compilePatternsAndApplyRegex">
    <!-- Controls that the pattern file is available and the input file can be re-loaded from fn:base-uri() -->
    <xsl:variable name="currentFileUri" select="base-uri()" as="xs:anyURI"/>
    <xsl:choose>
      <xsl:when test="not(unparsed-text-available($currentFileUri))">
        <xsl:call-template name="els:log">
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="code" select="'UNAVAILABLE_FILE'"/>
          <xsl:with-param name="xsltName" select="$resName"/>
          <xsl:with-param name="description">Problem while trying to read the file "<xsl:value-of select="$currentFileUri"/>" as unparsed-text.</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="not(unparsed-text-available($xfRes:TR_patternFileUri))">
        <xsl:call-template name="els:log">
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="code" select="'UNAVAILABLE_FILE'"/>
          <xsl:with-param name="xsltName" select="$resName"/>
          <xsl:with-param name="description">Problem while trying to read the file "<xsl:value-of select="$xfRes:TR_patternFileUri"/>" as unparsed-text.</xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <!-- STEP1 / Compile the patterns from the file ($xfRes:TR_patternFileUri) -->
        <xsl:variable name="step" as="element(patterns)">
          <patterns>
            <xsl:apply-templates select="unparsed-text-lines($xfRes:TR_patternFileUri,'UTF-8')" mode="xfRes:compilePatterns"/>
          </patterns>
        </xsl:variable>
        <xsl:if test="$xfRes:debug">
          <xsl:result-document method="xml" encoding="UTF-8" href="{resolve-uri(concat(els:getFileName($currentFileUri,false()),'.compiledPatterns.xml'),$currentFileUri)}">
            <xsl:sequence select="$step"/>
          </xsl:result-document>
        </xsl:if>
        <!-- STEP2 / Apply the regular expressions on the input file -->
        <xsl:variable name="step" as="xs:string">
          <xsl:value-of separator="">
            <xsl:call-template name="xfRes:applyRegex">
              <xsl:with-param name="str" tunnel="yes" select="unparsed-text($currentFileUri,'UTF-8')"/>
              <xsl:with-param name="regex.current" tunnel="yes" select="($step/pattern)[1]"/>
              <xsl:with-param name="regex.next" tunnel="yes" select="subsequence($step/pattern,2)"/>
            </xsl:call-template>
          </xsl:value-of>
        </xsl:variable>
        <xsl:if test="$xfRes:debug">
          <xsl:result-document method="text" encoding="UTF-8" href="{resolve-uri(concat(els:getFileName($currentFileUri,false()),'.textOutput.txt'),$currentFileUri)}">
            <xsl:sequence select="$step"/>
          </xsl:result-document>
        </xsl:if>
        <!-- STEP3 / Parses the string result and outputs it as XML -->
        <xsl:sequence select="parse-xml($step)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  
  <!--+==================================================+
      | MODE xfRes:compilePatterns                       |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>Parses a line in the pattern file.</xd:desc>
  </xd:doc>
  <xsl:template match="." mode="xfRes:compilePatterns">
    <!-- Separates the actual regex, the search flags and the capturing-groups attributes -->
    <xsl:analyze-string select="." regex="^/(.+?)/([a-z]*)\|\|\|\[(.+?)\]$">
      <xsl:matching-substring>
        <pattern>
          <!-- The regex can be broken into: optional context before, actual match, optional context after
               The actual pattern is inside parenthesis () and thus its matched string will always be in regex-group(1); contexts must not be placed in capturing groups
               Ex. : /the context before (the (matched) pattern) the context after/ -->
          <match>
            <xsl:value-of select="xfRes:normalizeXPathRegex(regex-group(1))"/>
          </match>
          <!-- Flags (except "g": no equivalent flag in XPath regex engine; hope this does not break anything...) -->
          <flags>
            <xsl:value-of select="replace(regex-group(2),'g','')"/>
            <!-- The 'j' flag in Saxon switches to the standard Java regular expression engine, allowing native Java regex syntax (including \b word boundaries) -->
            <xsl:text>;j</xsl:text>
          </flags>
          <!-- List of attributes to create from the capture-groups -->
          <captureGroups>
            <xsl:value-of select="regex-group(3)"/>
          </captureGroups>
        </pattern>
      </xsl:matching-substring>
      <xsl:non-matching-substring>
        <!-- Fallback: something is wrong with the pattern -->
        <xsl:call-template name="els:log">
          <xsl:with-param name="level" select="'error'"/>
          <xsl:with-param name="code" select="'UNRECOGNIZED_PATTERN_FORMAT'"/>
          <xsl:with-param name="xsltName" select="$resName"/>
          <xsl:with-param name="description">This pattern could not be parsed (check its format): <xsl:value-of select="."/></xsl:with-param>
        </xsl:call-template>
      </xsl:non-matching-substring>
    </xsl:analyze-string>
  </xsl:template>
  
  
  <!--+==================================================+
      | TEMPLATE xfRes:applyRegex                        |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>
      <xd:p>Templates that applies a regular expression derived from a pattern (<xd:ref name="regex.current" type="parameter">$regex.current</xd:ref>) to a string (<xd:ref name="str" type="parameter">$str</xd:ref>).</xd:p>
      <xd:p>A fn:analyze-string() function is build from the regex and applied dynamically.</xd:p>
      <xd:p>The result is a &lt;fn:analyze-string&gt; which is first reorganized to separate the contextual bits from the matched one.</xd:p>
      <xd:p>The matched bits are then processed and structured, and the next regular expression is applied on the remaining non-matching bits.</xd:p>
    </xd:desc>
    <xd:param name="str">[REQUIRED][TUNNEL][xs:string] A string on which to apply a regular expression.</xd:param>
    <xd:param name="regex.current">[REQUIRED][TUNNEL][element(pattern)] The compiled pattern which holds the regex to apply.</xd:param>
  </xd:doc>
  <xsl:template name="xfRes:applyRegex">
    <xsl:param name="str" tunnel="yes" required="yes" as="xs:string"/>
    <xsl:param name="regex.current" tunnel="yes" required="yes" as="element(pattern)"/>
    <!-- Constructs the XPath fn:analyze-string() function to be dynamically evaluated from $regex.current -->
    <xsl:variable name="match.function" select="concat('analyze-string(.,&quot;',$regex.current/match,'&quot;,&quot;',$regex.current/flags,'&quot;)')" as="xs:string"/>
    <!-- Analyzes the string $str according to $regex.current -->
    <!-- 1/ String analyzed with its contexts -->
    <xsl:variable name="match.contextualResult" as="element(fn:analyze-string-result)">
      <xsl:evaluate context-item="$str" xpath="$match.function"/>
    </xsl:variable>
    <!-- 2/ The contextual bits must be reintegrated inside the preceding/following non-match segments -->
    <xsl:variable name="match.result" as="element(fn:analyze-string-result)">
      <xsl:apply-templates select="$match.contextualResult" mode="xfRes:reorganizeMatchedContexts"/>
    </xsl:variable>
    <!-- 3/ Apply the next regex on the non-matched bits and process the matched ones -->
    <xsl:apply-templates select="$match.result" mode="xfRes:processRegexResult"/>
  </xsl:template>
  
  
  <!--+==================================================+
      | MODE xfRes:processRegexResult                    |
      +==================================================+-->
  
  <xsl:mode name="xfRes:processRegexResult" on-no-match="shallow-skip"/>
  
  <xd:doc>
    <xd:desc>Applies next regex on non-matched bits (or outputs them if there are no regex left).</xd:desc>
    <xd:param name="regex.next">[REQUIRED][TUNNEL][element(pattern)*] The compiled patterns which hold the next regex to apply.</xd:param>
  </xd:doc>
  <xsl:template match="fn:non-match" mode="xfRes:processRegexResult">
    <xsl:param name="regex.next" tunnel="yes" required="yes" as="element(pattern)*"/>
    <xsl:choose>
      <xsl:when test="not(empty($regex.next))">
        <xsl:call-template name="xfRes:applyRegex">
          <xsl:with-param name="str" tunnel="yes" select="."/>
          <xsl:with-param name="regex.current" tunnel="yes" select="$regex.next[1]"/>
          <xsl:with-param name="regex.next" tunnel="yes" select="subsequence($regex.next,2)"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Formats the matched bits according to the capture group names specified in the pattern.</xd:p>
    </xd:desc>
    <xd:param name="regex.current">[REQUIRED][TUNNEL][element(pattern)] The compiled pattern which holds the regex to apply.</xd:param>
  </xd:doc>
  <xsl:template match="fn:match" mode="xfRes:processRegexResult">
    <xsl:param name="regex.current" tunnel="yes" required="yes" as="element(pattern)"/>
    <xsl:variable name="self" select="." as="element(fn:match)"/>
    <xsl:text>&lt;a</xsl:text>
    <xsl:for-each select="tokenize($regex.current/captureGroups,',')">
      <xsl:variable name="position" select="position()" as="xs:integer"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="replace(.,concat('[',$els:quot,$els:dquot,']'),'')"/>
      <xsl:text>="</xsl:text>
      <xsl:value-of select="serialize($self//fn:group[@nr = ($position + 1)]/text())"/>
      <xsl:text>"</xsl:text>
    </xsl:for-each>
    <xsl:text>&gt;</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>&lt;/a&gt;</xsl:text>
  </xsl:template>
  
  
  <!--+==================================================+
      | MODE xfRes:reorganizeMatchedContexts             |
      +==================================================+-->
  
  <xsl:mode name="xfRes:reorganizeMatchedContexts" on-no-match="shallow-copy"/>
  
  <xd:doc>
    <xd:desc>Contextual bits (text() nodes not in the &lt;group nr="1"/&gt;) are deleted from match elements.</xd:desc>
  </xd:doc>
  <xsl:template match="fn:match/text()[. is parent::fn:match/child::node()[1]] |
                       fn:match/text()[. is parent::fn:match/child::node()[last()]]"
                mode="xfRes:reorganizeMatchedContexts"/>
  
  <xd:doc>
    <xd:desc>Adds the context before bit inside the preceding &lt;non-match&gt; element.</xd:desc>
  </xd:doc>
  <xsl:template match="fn:non-match[following-sibling::*[1]/self::fn:match]"
                mode="xfRes:reorganizeMatchedContexts"
                priority="2">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" mode="#current"/>
      <xsl:value-of select="following-sibling::*[1][self::fn:match]/node()[1][self::text()]"/>
    </xsl:copy>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Creates a &lt;non-match&gt; if there is none before the current &lt;match&gt; element and adds the context before bit inside.</xd:desc>
  </xd:doc>
  <xsl:template match="fn:match[not(preceding-sibling::*[1]/self::fn:non-match)]"
                mode="xfRes:reorganizeMatchedContexts"
                priority="2">
    <fn:non-match>
      <xsl:value-of select="node()[1][self::text()]"/>
    </fn:non-match>
    <xsl:next-match/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Adds the context after bit inside the following &lt;non-match&gt; element.</xd:desc>
  </xd:doc>
  <xsl:template match="fn:non-match[preceding-sibling::*[1]/self::fn:match]"
                mode="xfRes:reorganizeMatchedContexts"
                priority="1">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:value-of select="preceding-sibling::*[1][self::fn:match]/node()[last()][self::text()]"/>
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Creates a &lt;non-match&gt; if there is none after the current &lt;match&gt; element and adds the context after bit inside.</xd:desc>
  </xd:doc>
  <xsl:template match="fn:match[not(following-sibling::*[1]/self::fn:non-match)]"
                mode="xfRes:reorganizeMatchedContexts"
                priority="1">
    <xsl:next-match/>
    <fn:non-match>
      <xsl:value-of select="node()[last()][self::text()]"/>
    </fn:non-match>
  </xsl:template>
  
  
  <!--+==================================================+
      | FUNCTIONS                                        |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>Normalizes the input regex (Javascript format) to be compatible with XPath regex engine.</xd:desc>
    <xd:param name="regex">[xs:string] The input regex.</xd:param>
    <xd:return>[xs:string] The XPath normalized regex.</xd:return>
  </xd:doc>
  <xsl:function name="xfRes:normalizeXPathRegex" as="xs:string">
    <xsl:param name="regex" as="xs:string"/>
    <!-- / character must not be escaped -->
    <xsl:variable name="regex.tmp1" select="replace($regex,'([^\\])\\/','$1/')" as="xs:string"/>
    <!-- XPath does not recognized \u00A0 (non-breakable space) as part of \s character class, whereas Javascript does -->
    <xsl:variable name="regex.tmp2" select="replace($regex.tmp1,'\\s','[\\s ]')" as="xs:string"/>
    <xsl:sequence select="$regex.tmp2"/>
  </xsl:function>
  
</xsl:stylesheet>