<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
  xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst" 
  xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
  exclude-result-prefixes="#all"
  version="3.0">
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Check the required existance metadata of an EE according "ELS_META_txtClasseRefDoc" value</xd:p>
      <xd:p>2 steps :</xd:p>
      <xd:ul>
        <xd:li>Step 1 : check if there is at least one missing meta data using "xf:hasMinimumMetaData" function</xd:li>
        <xd:li>Step 2 : is case of missing meta data, display messages using "xf:getMessagesOfMissingMetas" message</xd:li>
      </xd:ul>
    </xd:desc>
  </xd:doc>
  
  <xsl:param name="xf:sch-srcUiTexte-debug" select="false()" as="xs:boolean"/>
  
  <xsl:import href="EFL_TEE_srcUiTexte.common.xsl"/>
  <!-- Import to change when xsl:import by CP fully works -->
  <xsl:import href="importToRemove/xf-lib/xf-common.xsl"/>
  <xsl:import href="importToRemove/xslLib/els-common_xml.xsl"/>

  <!-- Read the configuration file to manage systemTitle for EFL_TEE_srcUiTexte EE -->
  <xsl:variable name="xf:systemTitle.definitions" select="document(resolve-uri('conf/EFL_TEE_srcUiTexte.control.systemTitle.xml', static-base-uri()))" as="document-node()"/>  
  
  <!--==================================================-->
  <!-- INIT -->
  <!--==================================================-->
  <xd:doc>
    <xd:desc>
      <xd:p>Template main to use to check the XSL ONLY with oXygen; must be commented during schematon use</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="/*">
    <xsl:if test="not(xf:sch-srcUiTexte-hasMinimumMetaData(xf:metadata, xf:getMetaValue(xf:getMeta(., 'ELS_META_txtClasseRefDoc'))))">
      <xsl:sequence select="xf:sch-srcUiTexte-getMessagesOfMissingMetas(xf:metadata, xf:getMetaValue(xf:getMeta(., 'ELS_META_txtClasseRefDoc')))"></xsl:sequence>
    </xsl:if>
  </xsl:template>
  
  <!--==================================================-->
  <!-- Main mode="xf:sch-srcUiTexte-addExists" -->
  <!--==================================================-->
  <xd:doc>
    <xd:desc>
      <xd:p>Add @exists true() or false() at &lt;metaGroup&gt; and &lt;cle&gt; according the metadatas of the EE</xd:p>
    </xd:desc>
    <xd:param name="e.metadata.codes">sequence of string of the meta names</xd:param>
  </xd:doc>
  <xsl:template match="metaGroup | cle" mode="xf:sch-srcUiTexte-addExists">
    <xsl:param name="e.metadata.codes" as="xs:string*" tunnel="yes"/>
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:attribute name="exists" select="els:each-subsequence-item-included-into-sequence(descendant-or-self::cle/@code/string(), $e.metadata.codes)"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>  
  
  <!--==================================================-->
  <!-- Main mode="xf:sch-srcUiTexte-addCounters" -->
  <!--==================================================-->
  <xd:doc>
    <xd:desc>
      <xd:p>Add counters at the &lt;metaGroup&gt; and &lt;cle&gt; according the metadatas of the EE :</xd:p>
      <xd:ul>
        <xd:li>@nbrItem : nbr of item in the group</xd:li>
        <xd:li>@nbrMissedItem : nbr of missed item in the group</xd:li>
      </xd:ul>
    </xd:desc>
  </xd:doc>
  <xsl:template match="metaGroup[@exists=false()]" mode="xf:sch-srcUiTexte-addCounters">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:attribute name="nbrItem" select="count(cle)"/>
      <xsl:attribute name="nbrMissedItem" select="count(cle[@exists=false()])"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <!--==================================================-->
  <!-- Functions -->
  <!--==================================================-->  
  <xd:doc>
    <xd:desc>
      <xd:p>Main function to be called from the schematron to check if required metadatas is defined</xd:p>
      <xd:p>Required : The EE must contain "ELS_META_txtClasseRefDoc" metadata</xd:p>
      <xd:p>The minimum of metadata is calculated joining "refDocDefinitions.xml" and "EFL_TEE_srcUiTexte.control.systemTitle.xml" files</xd:p>      
      <xd:p>Function called from the schematron</xd:p>
    </xd:desc>
    <xd:param name="xf:metadata">xf:metadata of EE</xd:param>
    <xd:param name="classeRefDoc">classeRefDoc value of the EE</xd:param>
    <xd:return>true() is one group of metadata is confirmed otherwise returns false()</xd:return>
  </xd:doc>
  <xsl:function name="xf:sch-srcUiTexte-hasMinimumMetaData" as="xs:boolean">
    <xsl:param name="xf:metadata" as="element(xf:metadata)"/>
    <xsl:param name="classeRefDoc" as="xs:string"/>

    <!-- séquence of meta/@code of the EE -->
    <xsl:variable name="metadata.codes" select="distinct-values($xf:metadata/xf:meta[xf:hasContent(.)]/@code)" as="xs:string*"/>    
    
    <!-- Get the groups of meta available according @classeRefDoc value -->
    <xsl:variable name="xf:sch-srcUiTexte.metaGroups" as="element(metaGroups)">
      <xsl:call-template name="xf:sch-srcUiTexte-getMetadataGroups">
        <xsl:with-param name="refDocDefinition-elements" select="$xf:refDoc.definitions/refDocDefinitions/refDocDefinition[@classeRefDoc=$classeRefDoc]"/>
        <xsl:with-param name="systemTitleDefinition-elements" select="$xf:systemTitle.definitions/systemTitleDefinitions/systemTitleDefinition[@classeRefDoc=$classeRefDoc]"/>
      </xsl:call-template>
    </xsl:variable>    
    
    <!-- get all the @boolean of the checked metas -->    
    <xsl:variable name="hasNecessaryMetadatas" as="xs:boolean*">
      <xsl:for-each select="$xf:sch-srcUiTexte.metaGroups/metaGroup">
        <xsl:sequence select="els:each-subsequence-item-included-into-sequence(cle/@code/string(), $metadata.codes)"/>
      </xsl:for-each>
    </xsl:variable>
    
    <!-- Return true() if at least one metaGroup is available -->
    <xsl:sequence select="$hasNecessaryMetadatas = true()"/>
  </xsl:function>  

  <xd:doc>
    <xd:desc>
      <xd:p>Get the messages of missing metadata</xd:p>
      <xd:p>Function called from the schematron</xd:p>
    </xd:desc>
    <xd:param name="xf:metadata">xf:editorialEntity</xd:param>
    <xd:param name="classeRefDoc">classeRefDoc value</xd:param>
    <xd:return>All the message of missing metadatas according groups</xd:return>
  </xd:doc>
  <xsl:function name="xf:sch-srcUiTexte-getMessagesOfMissingMetas" as="xs:string*">
    <xsl:param name="xf:metadata" as="element(xf:metadata)"/> 
    <xsl:param name="classeRefDoc" as="xs:string"/>
        
    <!-- séquence of meta/@code of the EE -->
    <xsl:variable name="xf:metadata.codes" select="distinct-values($xf:metadata/xf:meta[xf:hasContent(.)]/@code)" as="xs:string*"/>        
    
    <!-- Get the groups of meta available according @classeRefDoc value -->
    <xsl:variable name="step" as="element(metaGroups)">
      <xsl:call-template name="xf:sch-srcUiTexte-getMetadataGroups">
        <xsl:with-param name="refDocDefinition-elements" select="$xf:refDoc.definitions/refDocDefinitions/refDocDefinition[@classeRefDoc=$classeRefDoc]"/>
        <xsl:with-param name="systemTitleDefinition-elements" select="$xf:systemTitle.definitions/systemTitleDefinitions/systemTitleDefinition[@classeRefDoc=$classeRefDoc]"/>
      </xsl:call-template>
    </xsl:variable>  
    
    <xsl:if test="$xf:sch-srcUiTexte-debug">
      <xsl:message>1:<xsl:copy-of select="$step"/></xsl:message>
    </xsl:if>
    
    <!-- Add @exists -->
    <xsl:variable name="step" as="element(metaGroup)*">
      <xsl:apply-templates select="$step/metaGroup" mode="xf:sch-srcUiTexte-addExists">
        <xsl:with-param name="e.metadata.codes" select="$xf:metadata.codes" tunnel="yes"/>
      </xsl:apply-templates>
    </xsl:variable>

    <xsl:if test="$xf:sch-srcUiTexte-debug">
      <xsl:message>2:<xsl:copy-of select="$step"/></xsl:message>      
    </xsl:if>
    
    <!-- Keep only the metaGroup[@exists=false()] -->
    <xsl:variable name="step" as="element(metaGroup)*">
      <xsl:copy-of select="$step[@exists=false()]"/>
    </xsl:variable>
    
    <xsl:if test="$xf:sch-srcUiTexte-debug">
      <xsl:message>3:<xsl:copy-of select="$step"/></xsl:message>
    </xsl:if>
    
    <!-- Add counters : @nbrItem and @nbrMissedItem -->
    <xsl:variable name="step" as="element(metaGroup)*">
      <xsl:apply-templates select="$step" mode="xf:sch-srcUiTexte-addCounters"/>
    </xsl:variable>
    
    <xsl:if test="$xf:sch-srcUiTexte-debug">
      <xsl:message>4:<xsl:copy-of select="$step"/></xsl:message>
    </xsl:if>
    
    <!-- Keep only the &lt;metaGroup&gt; which contains min of @nbrMissedItem-->
    <xsl:variable name="step" as="element(metaGroup)*">
      <xsl:copy-of select="$step[@nbrMissedItem = min($step/@nbrMissedItem)]"/>
    </xsl:variable>
    
    <xsl:if test="$xf:sch-srcUiTexte-debug">
      <xsl:message>5:<xsl:copy-of select="$step"/></xsl:message>
    </xsl:if>

    <!-- Keep only the 1st &lt;metaGroup&gt; which contains min of @nbrItem-->
    <xsl:variable name="step" as="element(metaGroup)">
      <xsl:copy-of select="$step[@nbrItem = min($step/@nbrItem)][1]"/>
    </xsl:variable>
    
    <xsl:if test="$xf:sch-srcUiTexte-debug">
      <xsl:message>6:<xsl:copy-of select="$step"/></xsl:message>
    </xsl:if>
    
    <!-- Create the message -->
    <xsl:variable name="sReturn" as="xs:string" select="concat(
        string-join(key('key.metadata.definitions', $step/cle[@exists=false()]/@code, $xf:metadata.definitions), ', '),
        ' - metadata(s) nécessaire(s) :',
        string-join(key('key.metadata.definitions', $step/cle/@code, $xf:metadata.definitions), ', ')
        )"/>            
    
    <!-- Send the message -->
    <xsl:value-of select="normalize-space($sReturn)"/>
  </xsl:function>  

  <xd:doc>
    <xd:desc>
      <xd:p>Calculate all the available sequences of codes according :</xd:p>
      <xd:ul>
        <xd:li>refDocDefinitions elements</xd:li>
        <xd:li>systemTitleDefinition elements</xd:li>
      </xd:ul>
    </xd:desc>
    <xd:param name="refDocDefinition-elements">list of &lt;refDocDefinition&gt;</xd:param>
    <xd:param name="systemTitleDefinition-elements">list of &lt;systemTitleDefinitions&gt;</xd:param>
    <xd:return>
      <xd:p>element &lt;metaGroups&gt; containing &lt;cle&gt; groups</xd:p>
      <xd:p>
        &lt;metaGroups&gt;
          &lt;metaGroup&gt;
            &lt;cle nom="xxx" code="ELS_META_xxx"/&gt;
            &lt;cle nom="xxx" code="ELS_META_xxx"/&gt;
            &lt;cle nom="xxx" code="ELS_META_xxx"/&gt;
          &lt;/metaGroup&gt;
          &lt;metaGroup&gt;
            &lt;cle nom="xxx" code="ELS_META_xxx"/&gt;
            &lt;cle nom="xxx" code="ELS_META_xxx"/&gt;          
          &lt;/metaGroup&gt;          
          ...
        &lt;/metaGroups&gt;
      </xd:p>
    </xd:return>
  </xd:doc>
  <xsl:template name="xf:sch-srcUiTexte-getMetadataGroups" as="element(metaGroups)">
    <xsl:param name="refDocDefinition-elements" as="element(refDocDefinition)*"/>
    <xsl:param name="systemTitleDefinition-elements" as="element(systemTitleDefinition)*"/>
    <!-- $list.one is the first non-empty list between $list.refDocDefinitions and $list.systemTitleDefinitions -->
    <xsl:variable name="list.one" select="if (empty($refDocDefinition-elements)) then $systemTitleDefinition-elements else $refDocDefinition-elements" as="element()*"/>
    <!-- Creation of sequence of &lt;metaGroup&gt; -->
    <xsl:variable name="metaGroup-elements" as="element(metaGroup)*">
      <!-- Nothing to do here is $list.one is empty -->
      <xsl:for-each select="$list.one">
        <!-- Current element in the first list -->
        <xsl:variable name="list.one.elt" select="." as="element()"/>
        <!-- The second list which can be obtained with a difference operation (can be empty) -->
        <xsl:variable name="list.two.tmp" select="($refDocDefinition-elements, $systemTitleDefinition-elements) except $list.one" as="element()*"/>
        <!-- If the second list is empty, we add the current element as "part of the second list" so that the following for-each is executed at least once -->
        <xsl:variable name="list.two" select="if (empty($list.two.tmp)) then ($list.one.elt) else ($list.two.tmp)" as="element()*"/>
        <xsl:for-each select="$list.two">
          <!-- $list.two.elt can be an element from the actual second list or the current element (in that case the cle elements will be deduplicated) -->
          <xsl:variable name="list.two.elt" select="." as="element()"/>
          <metaGroup>
            <!-- Remove duplicate cle elements from $list.one.elt and $list.two.elt -->           
            <!--<xsl:for-each select="distinct-values($list.one.elt/cle/@code[not(empty(.))] | $list.two.elt/cle/@code[not(empty(.))])">
              <xsl:variable name="codeValue" select="." as="xs:string"/>
              <xsl:copy-of select="($list.one.elt/cle[@code=$codeValue],$list.two.elt/cle[@code=$codeValue])[1]"/>
            </xsl:for-each>-->
            <xsl:copy-of select="$list.one.elt/cle"/>
            <xsl:copy-of select="$list.two.elt/cle[not(@code = $list.one.elt/cle/@code)]"/>
          </metaGroup> 
        </xsl:for-each>
      </xsl:for-each>
    </xsl:variable>
    <!-- Encapsulate &lt:metaGroup&gt; into a &lt;metaGroups&gt; element-->
    <metaGroups>
      <xsl:copy-of select="$metaGroup-elements"/>
    </metaGroups>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Check if there is a value in the &lt;xf:meta&gt;</xd:p>
    </xd:desc>
    <xd:param name="e">xf:meta element to check</xd:param>
    <xd:return>true() if there is one value (xf:value or xf:verbalization)</xd:return>
  </xd:doc>
  <xsl:function name="xf:hasContent" as="xs:boolean">
    <xsl:param name="e" as="element(xf:meta)"/>
    <xsl:sequence select="xf:getMetaReferenceItemRef($e) != '' or string-join(xf:getMetaValue($e), '') != ''"/>
</xsl:function>
      
  <xd:doc>
    <xd:desc>
      <xd:p>Default template modes : "xf:sch-srcUiTexte-addExists xf:sch-srcUiTexte-addCounters"</xd:p>
    </xd:desc>
  </xd:doc>
  <xsl:template match="node() | @*" mode="xf:sch-srcUiTexte-addExists xf:sch-srcUiTexte-addCounters">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
</xsl:stylesheet>