<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
                xmlns:xf="http://www.lefebvre-sarrut.eu/ns/xmlfirst"
                xmlns:ecm="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm"
                exclude-result-prefixes="#all"
                version="3.0">

  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>XSL used for Schematron about EE tree (revue, Fil d'Actus, etc.)</xd:p>
    </xd:desc>
  </xd:doc>
    
  <xd:doc>
    <xd:desc>ECM server URL (parameter provided by the ECM)</xd:desc>
  </xd:doc>
  <xsl:param name="ecm:server.url" select="''" as="xs:string"/>

  <xd:doc>
    <xd:desc>WS URL for editorialEntity serialization (calculated or possibly overridden if executed outside the ECM)</xd:desc>
  </xd:doc>
  <xsl:param name="ecm:ee.url.begin" select="concat($ecm:server.url, 'ecm-core/api/xmlExporter/exportToXML?flashId=')" as="xs:string"/>

  <!-- Import to change when xsl:import by CP fully works -->
  <xsl:import href="importToRemove/xslLib/els-common_convert-cast.xsl"/>
  <xsl:import href="importToRemove/xf-lib/xf-common.xsl"/>

  <xd:doc>
    <xd:desc>Fetches an editorialEntity using the ECM serialization WS.</xd:desc>
    <xd:param name="id">[xs:string?] The editorialEntity FLASH ID.</xd:param>
    <xd:return>[document-node()?] The editorialEntity, if found.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getEEFromId" as="document-node()?">
    <xsl:param name="id" as="xs:string?"/>
    <xsl:variable name="ee.uri" select="concat($ecm:ee.url.begin, $id)" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="doc-available($ee.uri)">
        <!-- EE is available ; retrive it -->
        <xsl:sequence select="doc($ee.uri)"/>
      </xsl:when>
      <xsl:otherwise>
        <!-- Error if EE is not found -->
        <xsl:message>[ERROR] EE with ID <xsl:value-of select="$id"/> could not be found (URI: <xsl:value-of select="$ee.uri"/>).</xsl:message>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <xd:doc>
    <xd:desc>
      <xd:p>Returns true() if the &lt;xf:structuralNode&gt; contains at least ONE meta "marqueur".</xd:p>
      <xd:p>(the meta is a referentiel meta))</xd:p>
    </xd:desc>
    <xd:param name="e">xf:structuralNode to check</xd:param>
  </xd:doc>
  <xsl:function name="xf:hasMetaMarqueur" as="xs:boolean">
    <xsl:param name="e" as="element(xf:structuralNode)"/>
    <!-- See the "referentiel_marqueur_global.xml" file in SAS project for EFL : &lt*:Referentiel_Nom&gt element -->
    <xsl:choose>
      <xsl:when test="$e/xf:metadata/xf:meta/xf:value/xf:ref[starts-with(@xf:targetResId, 'EFL_Marqueur')]">
        <!-- see the -->
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:when test="$e/xf:metadata/xf:meta/xf:value/xf:ref[@xf:targetResId = 'META_FLASH']">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <xd:doc>
    <xd:desc>
      <xd:p>Returns true() if the &lt;xf:structuralNode&gt; contains at the referential marqueur with [@idRef=$idRef] predicate.</xd:p>
      <xd:p>(the meta is a referentiel meta))</xd:p>
    </xd:desc>
    <xd:param name="e">xf:structuralNode to check</xd:param>
    <xd:param name="idRef">the @idRef to check</xd:param>
  </xd:doc>
  <xsl:function name="xf:hasReferentialMeta" as="xs:boolean">
    <xsl:param name="e" as="element(xf:structuralNode)"/>
    <xsl:param name="idRef" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="$e/xf:metadata/xf:meta/xf:value/xf:ref[@idRef = $idRef]">
        <xsl:value-of select="true()"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <xd:doc>
    <xd:desc>
      <xd:p>Returns true() if the teeCode is equals $teeCode</xd:p>
    </xd:desc>
    <xd:param name="e">Element of the "revue"</xd:param>
    <xd:param name="teeCode">Code of the TEE</xd:param>
  </xd:doc>
  <xsl:function name="xf:isCodeTee" as="xs:boolean">
    <xsl:param name="e" as="element()"/>
    <xsl:param name="teeCode" as="xs:string"/>
    <xsl:value-of select="$e/ancestor-or-self::xf:editorialEntity[last()]/@code = $teeCode"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Returns the EE "systemTitle" metadata value.</xd:p>
    </xd:desc>
    <xd:param name="e">[element(xf:editorialEntity)] The EE.</xd:param>
    <xd:return>[xs:string] The string value of the EE "systemTitle" metadata.</xd:return>
  </xd:doc>
  <xsl:function name="xf:getSystemTitle" as="xs:string?">
    <xsl:param name="e" as="element(xf:editorialEntity)?"/>
    <xsl:sequence select="xf:getMetaValue(xf:getMeta($e,'systemTitle'))"/>
  </xsl:function>
  
  <xd:doc>
    <xd:desc>
      <xd:p>Identifie si un referenceNode d'un Fil d'Actus doit être contrôlé, en fonction de critères éditoriaux.</xd:p>
    </xd:desc>
    <xd:param name="referenceNode">[element(xf:referenceNode)] Le referenceNode à analyser.</xd:param>
    <xd:return>[xs:boolean] Indique si les règles Schematron doivent s'appliquer au referenceNode.</xd:return>
  </xd:doc>
  <xsl:function name="xf:isRefNodeActuToBeChecked" as="xs:boolean">
    <xsl:param name="referenceNode" as="element(xf:referenceNode)"/>
    <xsl:choose>
      <!-- L'actualité n'a jamais été publiée -->
      <xsl:when test="empty(xf:getMeta($referenceNode,'publicationDate'))">
        <xsl:sequence select="true()"/>
      </xsl:when>
      <!-- Il y a une publicationDate mais le noeud d'accrochage a été modifié depuis -->
      <xsl:when test="$referenceNode/@lastModificationDateTime[. castable as xs:dateTime] cast as xs:dateTime
                      &gt;=
                      xf:getMetaDateTimeValue(xf:getMeta($referenceNode,'publicationDate'))">
        <xsl:sequence select="true()"/>
      </xsl:when>
      <!-- Il y a une publicationDate mais la date de publication suggérée est postérieure -->
      <xsl:when test="xf:getMetaDateTimeValue(xf:getMeta($referenceNode,'suggestedPublicationDate'))
                      &gt;=
                      xf:getMetaDateTimeValue(xf:getMeta($referenceNode,'publicationDate'))">
        <xsl:sequence select="true()"/>
      </xsl:when>
      <!-- Pas de contrôle dans les autres cas -->
      <xsl:otherwise>
        <xsl:sequence select="false()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
</xsl:stylesheet>
