<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
                xmlns:els="http://www.lefebvre-sarrut.eu/ns/els"
                xmlns:xfRes="http://www.lefebvre-sarrut.eu/ns/xmlfirst/ecm/resources" 
                exclude-result-prefixes="#all"
                version="3.0">
  
  
  <!--+==========================================================+
      | INIT.                                                    |
      +==========================================================+-->
  
  <!-- IMPORTS / INCLUDES -->
  <xsl:import href="../../../xslLib/xslLib/els-common.xsl"/>
  
  <xd:doc scope="stylesheet">
    <xd:desc>
      <xd:p>Génération d'un fichier de patterns de qualification automatique à partir d'une configuration XML.</xd:p>
      <xd:p>La configuration XML permet d'assembler des listes de termes pour former les patterns.</xd:p>
      <xd:p>L'objectif est d'améliorer la maintenabilité de ces patterns : on a d'un côté des listes correspondant à un type de donnée (ex. : des juridictions) et de l'autre un fichier qui décrit leur agencement.</xd:p>
      <xd:p>La XSL est exécutée lors du build Maven pour construire certains fichiers de patterns, qui du coup ne doivent pas être édités manuellement.</xd:p>
      <xd:p>Exemple de pattern décrit en XML :</xd:p>
      <xd:pre>
        <!-- @attributes contient la liste des attributs à créer, dans l'ordre de match -->
        <pattern attributes="data-xml-juridiction data-xml-lieu data-xml-date data-xml-numero">
          <!-- @idRef se rapport à l'ID d'une liste de termes -->
          <!-- Chaque <chunk> devient un groupe capturant ou non-capturant -->
          <chunk idRef="juri_lieu"/>
          <!-- Par défaut un <chunk> déclenche un match lié à un attribut (groupe capturant), sauf si @match="false" -->
          <chunk idRef="_sep" match="false"/>
          <chunk idRef="lieu"/>
          <!-- <group> permet de créer un groupe non-capturant -->
          <!-- @optional ajoute un ? d'optionalité -->
          <group optional="true">
            <chunk idRef="_sep" match="false"/>
            <chunk idRef="_precision" match="false"/>
          </group>
          <chunk idRef="_sep" match="false"/>
          <chunk idRef="_intro_date" match="false" optional="true"/>
          <chunk idRef="date"/>
          <chunk idRef="_sep" match="false"/>
          <chunk idRef="_intro_num" match="false"/>
          <!-- @optional + @multiple ajoute une * de 0 - infini -->
          <chunk idRef="_sep" match="false" optional="true" multiple="true"/>
          <chunk idRef="numero"/>
          <group optional="true">
            <chunk idRef="_sep" match="false"/>
            <chunk idRef="_precision" match="false"/>
          </group>
        </pattern>
      </xd:pre>
    </xd:desc>
  </xd:doc>
  
  <!-- PARAMS / VARIABLES -->
  <xsl:param name="xfRes:chunkUri" select="resolve-uri(concat($xfRes:fileNameBase,'.chunks/'),static-base-uri())" as="xs:anyURI"/>
  <xsl:variable name="xfRes:fileNameBase" select="substring-before(els:getFileName(base-uri(/),false()),'.conf')" as="xs:string"/>
  <xsl:variable name="xfRes:chunks" select="collection(concat($xfRes:chunkUri,'?select=*.xml'))/chunk" as="element(chunk)*"/>
  
  <!-- MODES -->
  <xsl:mode name="xfRes:makePatternFile" on-no-match="shallow-skip"/>
  
  <!-- OUTPUT -->
  <xsl:output method="text" encoding="UTF-8"/>
  
  <xd:doc>
    <xd:desc>Lancement sans mode</xd:desc>
  </xd:doc>
  <xsl:template match="/">
    <xsl:apply-templates select="." mode="xfRes:makePatternFile"/>
  </xsl:template>
  
  
  <!--+==================================================+
      | MAIN                                             |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>Matche un élément pattern décrivant une ligne dans le fichier de patterns à générer.</xd:desc>
  </xd:doc>
  <xsl:template match="pattern" mode="xfRes:makePatternFile">
    <xsl:text>/</xsl:text>
    <!-- Eventuel contexte avant le match -->
    <xsl:value-of select="@context-before"/>
    <xsl:text>(</xsl:text>
    <!-- Construction de l'expression régulière du pattern -->
    <xsl:apply-templates select="*" mode="#current"/>
    <xsl:text>)</xsl:text>
    <!-- Eventuel contexte après le match -->
    <xsl:value-of select="@context-after"/>
    <xsl:text>/</xsl:text>
    <!-- Flags pour le match (par défaut : "gi" - greedy + case insensitive) -->
    <xsl:value-of select="(@flags,'gi')[1]"/>
    <xsl:text>|||[</xsl:text>
    <!-- Liste des attributs à créer, cf. groupes de capture -->
    <xsl:value-of select="string-join(tokenize(@attributes,'\s+')!concat($els:quot,.,$els:quot),',')"/>
    <xsl:text>]</xsl:text>
    <!-- Saut de ligne entre patterns -->
    <xsl:if test="following-sibling::pattern">
      <xsl:value-of select="'&#10;'"/>
    </xsl:if>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Décrit un bout de l'expression régulière du pattern, dont le contenu doit être généré à partir d'une liste de termes.</xd:desc>
  </xd:doc>
  <xsl:template match="chunk" mode="xfRes:makePatternFile">
    <!-- Parenthèse ouvrante -->
    <xsl:call-template name="xfRes:makeGroupBeforeContext">
      <xsl:with-param name="xfRes:defaultMatch" select="true()" as="xs:boolean"/>
    </xsl:call-template>
    <!-- Récupère le contenu à générer en lisant la liste de termes dont l'ID correspond à @idRef -->
    <xsl:apply-templates select="$xfRes:chunks[@id = current()/@idRef]/entry" mode="#current"/>
    <!-- Parenthèse fermante -->
    <xsl:call-template name="xfRes:makeGroupAfterContext"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Erreur en cas d'appel à un fichier de termes inconnu.</xd:desc>
  </xd:doc>
  <xsl:template match="chunk[not(some $id in $xfRes:chunks/@id satisfies $id = @idRef)]" mode="xfRes:makePatternFile" priority="1">
    <xsl:message terminate="yes">[ERROR] No regex chunk file matches the following ID: <xsl:value-of select="@idRef"/>.</xsl:message>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Bout d'expression régulière directement intégré dans le fichier de conf. Utile pour des petits bouts de code non-réutilisables. Le contenu textuel de l'élément est directement utilisé.</xd:desc>
  </xd:doc>
  <xsl:template match="string" mode="xfRes:makePatternFile">
    <xsl:value-of select="."/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Crée un groupe capturant / non-capturant autour de certains chunks.</xd:desc>
  </xd:doc>
  <xsl:template match="group" mode="xfRes:makePatternFile">
    <!-- Parenthèse ouvrante -->
    <xsl:call-template name="xfRes:makeGroupBeforeContext"/>
    <!-- Contenu du groupe -->
    <xsl:apply-templates mode="#current"/>
    <!-- Parenthèse fermante -->
    <xsl:call-template name="xfRes:makeGroupAfterContext"/>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Introduit une alternative entre plusieurs groupes ou chunks.</xd:desc>
  </xd:doc>
  <xsl:template match="choice" mode="xfRes:makePatternFile">
    <!-- Parenthèse ouvrante non-capturante -->
    <xsl:text>(?:</xsl:text>
    <!-- Contenu du groupe -->
    <xsl:apply-templates mode="#current"/>
    <!-- Simple parenthèse fermante -->
    <xsl:text>)</xsl:text>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Permet l'insertion du caractère pipe "|" entre les membres d'un choice.</xd:desc>
  </xd:doc>
  <xsl:template match="choice/*" mode="xfRes:makePatternFile" priority="2">
    <!-- Contenu de l'élément -->
    <xsl:next-match/>
    <!-- Insère un pipe de choix s'il y a des following-siblings -->
    <xsl:if test="following-sibling::*">
      <xsl:text>|</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Une entrée dans une liste de termes. Toutes les entrées d'une liste sont concaténées dans un groupe (capturant ou non) et séparées par des pipes "|".</xd:desc>
  </xd:doc>
  <xsl:template match="entry" mode="xfRes:makePatternFile">
    <xsl:value-of select="."/>
    <xsl:if test="following-sibling::entry">
      <xsl:text>|</xsl:text>
    </xsl:if>
  </xsl:template>
  
  
  <!--+==================================================+
      | TEMPLATES UTILITAIRES / FONCTIONS                |
      +==================================================+-->
  
  <xd:doc>
    <xd:desc>Gère l'écriture de la parenthèse ouvrante d'un groupe, capturant ou non-capturant.</xd:desc>
    <xd:param name="xfRes:defaultMatch">[xs:boolean][OPTIONAL] Détermine si le groupe est capturant par défaut en l'absence d'attribut @match (valeur par défaut : false()).</xd:param>
  </xd:doc>
  <xsl:template name="xfRes:makeGroupBeforeContext">
    <xsl:param name="xfRes:defaultMatch" required="no" select="false()" as="xs:boolean"/>
    <xsl:text>(</xsl:text>
    <!-- Groupe non-capturant selon @match et $xfRes:defaultMatch -->
    <xsl:if test="(not(@match) and $xfRes:defaultMatch = false()) or (@match = 'false')">
      <xsl:text>?:</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xd:doc>
    <xd:desc>Gère l'écriture de la parenthèse fermante d'un groupe, et son éventuel caractère facultatif / récursif.</xd:desc>
  </xd:doc>
  <xsl:template name="xfRes:makeGroupAfterContext">
    <xsl:text>)</xsl:text>
    <!-- Optionalité + récursion -->
    <xsl:choose>
      <xsl:when test="@optional = 'true' and @multiple = 'true'">
        <xsl:text>*</xsl:text>
      </xsl:when>
      <xsl:when test="@optional = 'false' and @multiple = 'true'">
        <xsl:text>+</xsl:text>
      </xsl:when>
      <xsl:when test="@optional = 'true'">
        <xsl:text>?</xsl:text>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  
</xsl:stylesheet>